@@ -26,36 +26,40 @@
 
 static void read_value (GdbInstance *inst, GDF_Symbol *sym);
 static void check_type (GDF_Symbol *sym);
-static void expand_pointer (GdbInstance *inst, GdfLibdryadDebugger *dbg,
+static void expand_pointer (GdbInstance *inst, GdfLibdryadSymbolSet *ss,
 			    GDF_Symbol *sym);
-static void expand_structure (GdbInstance *inst, GdfLibdryadDebugger *dbg, 
+static void expand_structure (GdbInstance *inst, GdfLibdryadSymbolSet *ss, 
 			      GDF_Symbol *sym);
 
-static GDF_SymbolSet *init_symset (void);
+static GDF_SymbolSequence *init_seq (void);
+static void grow_seq (GDF_SymbolSequence *seq, int amount);
 
-static void grow_symset (GDF_SymbolSet *symset, int amount);
-
-GDF_SymbolSet *
-gdb_get_locals (GdbInstance *inst, GdfLibdryadDebugger *dbg)
+void
+gdb_get_locals (GdbInstance *inst, GdfLibdryadSymbolSet *ss)
 {
-    GDF_SymbolSet *symset;
     char *line;
     GList *names = NULL;
     GList *i;
     char *work;
+
+    if (ss->symbols) {
+	CORBA_free (ss->symbols);
+    }
+    ss->set_size = 0;
     
     gdb_execute_command (inst, "info locals");
 
-    symset = gdb_allocate_symbols ();
+    ss->symbols = gdb_allocate_symbols ();
 
+    /* Read the names of all the locals */
     line = gdb_read_line (inst);
     while (strcmp (line, "\32\32pre-prompt")) {
 	if ((work = strstr (line, " = "))) {
 	    *work = '\0';
 	    work += 3;
-
+	    
 	    names = g_list_prepend (names, g_strdup (line));
-
+	    
 	    if (!strncmp (work, "{", 1)) {
 		int levels = 1;
 		while (levels > 0) {
@@ -70,28 +74,29 @@ gdb_get_locals (GdbInstance *inst, GdfLibdryadDebugger *dbg)
 	line = gdb_read_line (inst);
     }
 
+
+    /* Add all the locals to the symbol set */
     names = g_list_reverse (names);
 
     for (i = names; i != NULL; i = i->next) {
-	gdb_add_expression (inst, dbg, symset, GDF_SYMBOL_SET_LOCALS, 
-			    i->data);
+	gdb_add_expression (inst, ss, i->data);
 	g_free (i->data);
     }
     g_list_free (names);
 
-    return symset;
+    /* Notify the symbol set that it has changed */
+    gdf_libdryad_symbol_set_changed (ss);
 }
 
 void
-gdb_get_expression (GdbInstance *inst, GdfLibdryadDebugger *dbg, 
-		    GDF_Symbol *sym, const char *expr)
+gdb_get_expression (GdbInstance *inst, GDF_Symbol *sym, const char *expr)
 {
     char *cmd_str;
     
     cmd_str = g_strdup_printf ("output %s", expr);
     gdb_execute_command (inst, cmd_str);
     g_free (cmd_str);
-
+    
     sym->name = CORBA_string_dup (expr);
     sym->expression = CORBA_string_dup (expr);
     read_value (inst, sym);
@@ -99,37 +104,30 @@ gdb_get_expression (GdbInstance *inst, GdfLibdryadDebugger *dbg,
 }
 
 void
-gdb_expand_symbol (GdbInstance *inst, GdfLibdryadDebugger *dbg, 
+gdb_expand_symbol (GdbInstance *inst, GdfLibdryadSymbolSet *ss, 
 		   GDF_Symbol *sym)
 {
-    CORBA_any *event_any;
-    GList *args = NULL;
-
     g_return_if_fail (sym->expandable);
 
-    switch (sym->type) {
-    case GDF_TYPE_PTR :
-	expand_pointer (inst, dbg, sym);
-	break;
-    case GDF_TYPE_STRUCT :
-	expand_structure (inst, dbg, sym);
-	break;
-    default:
-	return;
+    if (!sym->expanded) {
+	switch (sym->type) {
+	case GDF_TYPE_PTR :
+	    expand_pointer (inst, ss, sym);
+	    break;
+	case GDF_TYPE_STRUCT :
+	    expand_structure (inst, ss, sym);
+	    break;
+	default:
+	    return;
+	}
     }
 
     sym->expanded = TRUE;
-    args = g_list_append (NULL, GINT_TO_POINTER (sym->symbol_set));
-    args = g_list_append (args, GINT_TO_POINTER (sym->handle));
-    event_any = gdf_marshal_event_long_list ("symbol_expanded", args);	    
-    gdf_event_channel_client_push (dbg->event_channel, event_any);
-    CORBA_free (event_any);
-    g_list_free (args);
 }
 
-void
-gdb_update_symbols (GdbInstance *inst, GdfLibdryadDebugger *dbg, 
-		    GDF_SymbolSet *symset)
+static void
+update_seq (GdbInstance *inst, GdfLibdryadSymbolSet *ss, 
+	    GDF_SymbolSequence *seq) 
 {
     char *cmd_str;
     char *old_value;
@@ -138,8 +136,8 @@ gdb_update_symbols (GdbInstance *inst, GdfLibdryadDebugger *dbg,
 
     int i;
 
-    for (i = 0; i < symset->_length; i++) {
-	sym = &symset->_buffer[i];
+    for (i = 0; i < seq->_length; i++) {
+	sym = &seq->_buffer[i];
 	
 	cmd_str = g_strdup_printf ("output %s", sym->expression);
 	gdb_execute_command (inst, cmd_str);
@@ -154,68 +152,55 @@ gdb_update_symbols (GdbInstance *inst, GdfLibdryadDebugger *dbg,
 	if (strcmp (old_value, sym->value) 
 	    || was_expandable != sym->expandable) {
 	    CORBA_any *event_any;
-	    GList *args = NULL;
 
-	    args = 
-		g_list_append (args, GINT_TO_POINTER (sym->symbol_set));
-	    args = 
-		g_list_append (args, GINT_TO_POINTER (sym->handle));
+	    event_any = gdf_marshal_event_long ("symbol_changed", 
+						sym->handle);
 	    
-	    event_any = gdf_marshal_event_long_list ("symbol_changed", 
-						     args);
-	    
-	    g_list_free (args);
-	    
-	    if (dbg->event_queue) {
-		dbg->event_queue = g_list_prepend (dbg->event_queue,
-						    event_any);
-	    } else {
-		gdf_event_channel_client_push (dbg->event_channel,
-					       event_any);
-		CORBA_free (event_any);
-	    }
+	    gdf_event_channel_client_push (ss->event_channel,
+					   event_any);
+	    CORBA_free (event_any);
 	}
 	
 	CORBA_free (old_value);
-
-	gdb_update_symbols (inst, dbg, 
-			    (GDF_SymbolSet*)sym->children._value);
+	
+	update_seq (inst, ss, 
+		    (GDF_SymbolSequence*)sym->children._value);
     }
 }
 
-GDF_SymbolSet *
+void
+gdb_update_symbols (GdbInstance *inst, GdfLibdryadSymbolSet *ss)
+{
+    update_seq (inst, ss, ss->symbols);
+}
+
+GDF_SymbolSequence *
 gdb_allocate_symbols (void)
 {
-    return init_symset ();
+    return init_seq ();
+}
+
+static void
+add_expression_to_seq  (GdbInstance *inst,
+			GDF_SymbolSequence *seq, 
+			GdfLibdryadSymbolSet *ss, 
+			const char *expr)
+{
+    GDF_Symbol *sym;
+    
+    grow_seq (seq, 1);
+    sym = &seq->_buffer[seq->_length - 1];
+    gdb_get_expression (inst, sym, expr);
+    
+    sym->handle = ss->set_size++;
 }
 
 void
 gdb_add_expression (GdbInstance *inst,
-		    GdfLibdryadDebugger *dbg, 
-		    GDF_SymbolSet *symset, 
-		    int symset_id, const char *expr)
+		    GdfLibdryadSymbolSet *ss, 
+		    const char *expr)
 {
-    int set_size;
-    GDF_Symbol *sym;
-
-    grow_symset (symset, 1);
-    sym = &symset->_buffer[symset->_length - 1];
-    gdb_get_expression (inst, dbg, sym, expr);
-
-    set_size = GPOINTER_TO_INT (g_tree_lookup (dbg->sym_set_sizes,
-				GINT_TO_POINTER (symset_id)));
-    sym->symbol_set = symset_id;
-    sym->handle = set_size++;
-    g_tree_insert (dbg->sym_set_sizes, GINT_TO_POINTER (symset_id), 
-		   GINT_TO_POINTER (set_size));
-}
-
-
-
-void
-gdb_destroy_symbols (GDF_SymbolSet *syms)
-{
-    CORBA_free (syms);
+    add_expression_to_seq (inst, ss->symbols, ss, expr);
 }
 
 void
@@ -227,14 +212,29 @@ read_value (GdbInstance *inst, GDF_Symbol *sym)
     line = gdb_read_line (inst);
     while (strcmp (line, "\32\32pre-prompt")) {
 	if (!strncmp(line, "\32\32value-begin", strlen("\32\32value-begin"))) {
-	    /* FIXME: Deal with multiline values */
 	    line = gdb_read_line (inst);
 	    
-	    if (line[0] == '\0') 
-		sym->value = CORBA_string_dup ("Error reading value");
-	    else 
-		sym->value = CORBA_string_dup (line);
+	    while (line[0] == '\0' && strcmp (line, "\32\32\error-begin")) {
+		line = gdb_read_line (inst);
+	    }
 	    
+	    if (strcmp (line, "\32\32error-begin")) {
+		sym->value = CORBA_string_dup (line);
+	    } else {
+		while (strcmp (line, "\32\32error")) {
+		    char *new;
+		    line = gdb_read_line (inst);
+		    new = g_strdup_printf ("%s%s",
+					   sym->value ? sym->value : "", 
+					   line);
+		    if (sym->value) {
+			CORBA_free (sym->value);
+		    }
+		    sym->value = CORBA_string_dup (new);
+		    g_free (new);
+		}
+		break;
+	    }
 
 	    if (!strncmp (line, "{", 1)) {
 		int levels = 1;
@@ -249,13 +249,18 @@ read_value (GdbInstance *inst, GDF_Symbol *sym)
 	    break;
 	}
 	if (!strcmp (line, "\32\32error-begin")) {
-	    /* FIXME: Deal with multi-line error-messages */
-	    line = gdb_read_line (inst);
-	    sym->value = CORBA_string_dup (line);
 	    while (strcmp (line, "\32\32error")) {
+		char *new;
 		line = gdb_read_line (inst);
+		new = g_strdup_printf ("%s%s",
+				       sym->value ? sym->value : "", 
+					   line);
+		if (sym->value) {
+		    CORBA_free (sym->value);
+		}
+		sym->value = CORBA_string_dup (new);
+		g_free (new);
 	    }
-	    break;
 	}
 	line = gdb_read_line (inst);
     }
@@ -287,43 +292,43 @@ check_type (GDF_Symbol *sym)
 	sym->type = GDF_TYPE_STRUCT;
 	return;
     }
-    /* FIXME: Check for struct types */
     /* FIXME: Check for array types */
 
     sym->type = GDF_TYPE_NORMAL;
 }
 
 static void
-add_child_expression (GdbInstance *inst, GdfLibdryadDebugger *dbg, 
+add_child_expression (GdbInstance *inst, GdfLibdryadSymbolSet *ss, 
 		      GDF_Symbol *sym, const char *name, const char *expr)
 {
     GDF_Symbol *new_sym;
-    GDF_SymbolSet *children;
+    GDF_SymbolSequence *children;
 
     children = sym->children._value;
-    gdb_add_expression (inst, dbg, children, sym->symbol_set, expr);
+    add_expression_to_seq (inst, children, ss, expr);
     new_sym = &(children->_buffer[children->_length - 1]);
     CORBA_free (new_sym->name);
     new_sym->name = CORBA_string_dup (name);
 }
 
 void
-expand_pointer (GdbInstance *inst, GdfLibdryadDebugger *dbg, GDF_Symbol *sym)
+expand_pointer (GdbInstance *inst, GdfLibdryadSymbolSet *ss, 
+		GDF_Symbol *sym)
 {
     char *new_name;
     char *new_expr;
     new_name = g_strdup_printf ("*%s", sym->name);
     new_expr = g_strdup_printf ("*(%s)", sym->expression);
-    add_child_expression (inst, dbg, sym, new_name, new_expr);
+    add_child_expression (inst, ss, sym, new_name, new_expr);
     g_free (new_expr);
     g_free (new_name);
 }
 
 void
-expand_structure (GdbInstance *inst, GdfLibdryadDebugger *dbg, 
+expand_structure (GdbInstance *inst, GdfLibdryadSymbolSet *ss, 
 		  GDF_Symbol *sym)
 {
-    GDF_SymbolSet *children;
+    GDF_SymbolSequence *children;
     char *expr;
     char *cmd_str;
     char *line;
@@ -358,44 +363,46 @@ expand_structure (GdbInstance *inst, GdfLibdryadDebugger *dbg,
 
     names = g_list_reverse (names);
     for (i = names; i != NULL; i = i->next) {
+	g_assert (i != NULL);
 	expr = g_strdup_printf ("(%s).%s", sym->expression, (char*)i->data);
-	add_child_expression (inst, dbg, sym, i->data, expr);
+	g_assert (i != NULL);
+	add_child_expression (inst, ss, sym, i->data, expr);
 	g_free (expr);
 	g_free (i->data);
     }
     g_list_free (names); 
 }
 
-static GDF_SymbolSet *
-init_symset (void)
+static GDF_SymbolSequence *
+init_seq (void)
 {
-    GDF_SymbolSet *symset = GDF_SymbolSet__alloc ();
-    symset->_length = 0;
-    symset->_maximum = 0;
-    symset->_buffer = NULL;
+    GDF_SymbolSequence *seq = GDF_SymbolSequence__alloc ();
+    seq->_length = 0;
+    seq->_maximum = 0;
+    seq->_buffer = NULL;
     /* We tell it not to release now, but will set release to TRUE when the
      * buffer is grown for the first time */
-    CORBA_sequence_set_release (symset, CORBA_FALSE);
-    return symset;
+    CORBA_sequence_set_release (seq, CORBA_FALSE);
+    return seq;
 }
 
 void
-grow_symset (GDF_SymbolSet *symset, int amount)
+grow_seq (GDF_SymbolSequence *seq, int amount)
 {
     int i;
 
-    int old_max = symset->_maximum;
-    GDF_Symbol *old_buf = symset->_buffer;
+    int old_max = seq->_maximum;
+    GDF_Symbol *old_buf = seq->_buffer;
 
-    symset->_maximum += amount;
-    symset->_length += amount;
-    symset->_buffer = CORBA_sequence_GDF_Symbol_allocbuf (symset->_maximum);
+    seq->_maximum += amount;
+    seq->_length += amount;
+    seq->_buffer = CORBA_sequence_GDF_Symbol_allocbuf (seq->_maximum);
 
     if (old_buf) {
 	ORBit_mem_info *block;
 
 	for (i = 0; i < old_max; i++) {
-	    GDF_Symbol *sym = &(symset->_buffer[i]);
+	    GDF_Symbol *sym = &(seq->_buffer[i]);
 	    GDF_Symbol *old_sym = &old_buf[i];
 	    sym->symbol_set = old_sym->symbol_set;
 	    sym->handle = old_sym->handle;
@@ -418,18 +425,18 @@ grow_symset (GDF_SymbolSet *symset, int amount)
     } else {
 	/* _release was previously set to FALSE (because there was no buffer).
 	 * set it to true now that a buffer has been created. */
-	CORBA_sequence_set_release (symset, CORBA_TRUE);
+	CORBA_sequence_set_release (seq, CORBA_TRUE);
     }
 
-    for (i = symset->_length - amount; i < symset->_length; i++) {
-	GDF_Symbol *sym = &(symset->_buffer[i]);
+    for (i = seq->_length - amount; i < seq->_length; i++) {
+	GDF_Symbol *sym = &(seq->_buffer[i]);
 	sym->symbol_set = -1;
 	sym->handle = -1;
 	sym->type = GDF_TYPE_NORMAL;
 	sym->expandable = CORBA_FALSE;
 	sym->expanded = CORBA_FALSE;
 	sym->children._type = TC_CORBA_sequence_GDF_Symbol;
-	sym->children._value = init_symset ();
+	sym->children._value = init_seq ();
 	CORBA_any_set_release (&sym->children, CORBA_TRUE);
     }
 }
