@@ -36,24 +36,13 @@
 #include "libdryad.h"
 #include "../../lib/gnome-debug.h"
 
-typedef struct 
-{
-    GNode *syms;
-    GDF_SymbolSet *corba_syms;
-    int num_syms;
-    int set_id;
-} GdfSymSet;
 
 static CORBA_Object create_libdryad_debugger (BonoboObject *object);
 static void gdf_libdryad_debugger_destroy (GtkObject *object);
 static void gdf_libdryad_debugger_class_init (GdfLibdryadDebuggerClass *class);
 static void gdf_libdryad_debugger_init (BonoboObject *object);
 static void connect_signals (GdfLibdryadDebugger *dbg);
-static int int_cmp (gconstpointer a, gconstpointer b);
 static void process_frame_change (GdfLibdryadDebugger *dbg);
-static GDF_Symbol *find_sym_by_id (GdfLibdryadDebugger *dbg, 
-				   GDF_SymbolSetId set, 
-				   int sym_id);
 
 static BonoboObjectClass *parent_class;
 static POA_GDF_Debugger__epv libdryad_debugger_epv;
@@ -94,8 +83,9 @@ gdf_libdryad_debugger_new (void)
     debugger->pending_exit = -1;
 
     debugger->stopped_handlers = NULL;
-    debugger->sym_sets = g_tree_new (int_cmp);
-    debugger->sym_set_sizes = g_tree_new (int_cmp);
+    debugger->locals = gdf_libdryad_symbol_set_new (debugger);
+    debugger->symsets = NULL;
+    
     debugger->frame_changed = FALSE;
     
     return debugger;
@@ -666,223 +656,45 @@ impl_step_out (PortableServer_Servant servant, CORBA_Environment *ev)
 
 /* Symbol watching */
 
-static gint 
-update_traverse_func (gpointer key, gpointer value, gpointer data)
-{
-    GdfLibdryadDebugger *dbg = data;
-   
-    gdb_update_symbols (dbg->gdb_inst, dbg, (GDF_SymbolSet*)value);
-    return FALSE;
-}
-
-static gint
-update_traverse_func_no_locals (gpointer key, gpointer value, gpointer data)
-{
-    if (GPOINTER_TO_INT (key) == GDF_SYMBOL_SET_LOCALS)
-	return FALSE;
-
-    return update_traverse_func (key, value, data);
-}
-
 static void
 update_symsets (GdfLibdryadDebugger *dbg, gboolean include_locals)
 {
+    GList *i;
+    
     if (include_locals) {
-	g_tree_traverse (dbg->sym_sets, update_traverse_func,
-			 G_IN_ORDER, dbg);
-    } else {
-	g_tree_traverse (dbg->sym_sets, update_traverse_func_no_locals,
-			 G_IN_ORDER, dbg);
+	gdf_libdryad_symbol_set_update (dbg->locals);
+    }
+
+    for (i = dbg->symsets; i != NULL; i = i->next) {
+	gdf_libdryad_symbol_set_update (i->data);
     }
 }
 
 static void
 new_locals (GdfLibdryadDebugger *dbg)
 {
-    GDF_SymbolSet *symset;
-    CORBA_any *event_any;
-
-    symset = g_tree_lookup (dbg->sym_sets, 
-			    GINT_TO_POINTER (GDF_SYMBOL_SET_LOCALS));
-    
-    if (symset) {
-	CORBA_free (symset);
-    }
-
-    symset = gdb_get_locals (dbg->gdb_inst, dbg);
-    g_tree_insert (dbg->sym_sets, GINT_TO_POINTER (GDF_SYMBOL_SET_LOCALS),
-		   symset);
-
-    event_any = gdf_marshal_event_long ("symbol_set_changed",
-					(long)GDF_SYMBOL_SET_LOCALS);
-
-    if (dbg->event_queue) {
-	dbg->event_queue = g_list_prepend (dbg->event_queue, event_any);
-    } else {
-	gdf_event_channel_client_push (dbg->event_channel, event_any);
-	CORBA_free (event_any);
-    }
+    gdb_get_locals (dbg->gdb_inst, dbg->locals);
 }
 
-static CORBA_long 
+static GDF_SymbolSet
+impl_get_locals (PortableServer_Servant servant, CORBA_Environment *ev)
+{
+    GdfLibdryadDebugger *dbg = libdryad_debugger_from_servant (servant);
+    return CORBA_Object_duplicate (bonobo_object_corba_objref (BONOBO_OBJECT (dbg->locals)), 
+				   ev);
+}
+
+static GDF_SymbolSet
 impl_allocate_symbol_set (PortableServer_Servant servant, 
 			  CORBA_Environment *ev)
-{
-    static int symset_id = 1;  /* skip 0, which is the pre-defined locals 
-				* symbol set */
-    
+{    
     GdfLibdryadDebugger *dbg = libdryad_debugger_from_servant (servant);
-    GDF_SymbolSet *symset;
+    GdfLibdryadSymbolSet *ss = gdf_libdryad_symbol_set_new (dbg);
 
-    symset = gdb_allocate_symbols ();
-    
-    g_tree_insert (dbg->sym_sets, GINT_TO_POINTER (symset_id), symset);
+    dbg->symsets = g_list_prepend (dbg->symsets, ss);
 
-    return (CORBA_long)symset_id++;
-}
-
-static void
-impl_destroy_symbol_set (PortableServer_Servant servant,
-			 CORBA_long id,
-			 CORBA_Environment *ev)
-{
-    GdfLibdryadDebugger *dbg = libdryad_debugger_from_servant (servant);
-    GDF_SymbolSet *symset;
-
-    symset = g_tree_lookup (dbg->sym_sets, GINT_TO_POINTER ((int)id));
-    
-    g_return_if_fail (symset != NULL);
-    
-    gdb_destroy_symbols (symset);
-
-    g_tree_remove (dbg->sym_sets, GINT_TO_POINTER ((int)id));
-}
-
-static GDF_SymbolSet *
-impl_get_symbol_set (PortableServer_Servant servant, 
-		     CORBA_long id,
-		     CORBA_Environment *ev)
-{
-    GDF_SymbolSet *symset;
-    GDF_SymbolSet *ret;
-    GdfLibdryadDebugger *dbg = libdryad_debugger_from_servant (servant);
-
-    GDF_TRACE ();
-
-    symset = g_tree_lookup (dbg->sym_sets, GINT_TO_POINTER (id));
-    ret = GDF_SymbolSet__alloc ();
-
-    if (symset != NULL) {
-	ret->_length = symset->_length;
-	ret->_maximum = symset->_maximum;
-	ret->_buffer = symset->_buffer;
-	CORBA_sequence_set_release (ret, CORBA_FALSE);
-    } else {
-	ret->_length = 0;
-	ret->_maximum = 0;
-	ret->_buffer = NULL;
-    }
-
-    GDF_TRACE ();
- 
-    return ret;
-}
-
-static void
-impl_add_to_symbol_set (PortableServer_Servant servant,
-			CORBA_long set,
-			const CORBA_char *expr,
-			CORBA_Environment *ev)
-{
-    GdfLibdryadDebugger *dbg = libdryad_debugger_from_servant (servant);
-    GDF_SymbolSet *symset;
-    CORBA_any *event_any;
-    
-    symset = g_tree_lookup (dbg->sym_sets, GINT_TO_POINTER (set));
-    gdb_add_expression (dbg->gdb_inst, dbg, symset, set, expr);
-    
-    event_any = gdf_marshal_event_long ("symbol_set_changed", (long)set);
-    gdf_event_channel_client_push (dbg->event_channel, event_any);
-    CORBA_free (event_any);
-}
-
-static GDF_Symbol *
-impl_get_symbol (PortableServer_Servant servant,
-		 GDF_SymbolSetId set,
-		 CORBA_long id,
-		 CORBA_Environment *ev)
-{
-    GDF_Symbol *ret;
-    GDF_Symbol *sym;
-    GdfLibdryadDebugger *dbg = libdryad_debugger_from_servant (servant);
-
-    sym = find_sym_by_id (dbg, set, id);
-    g_assert (sym);
-    
-    ret = GDF_Symbol__alloc ();
-    ret->symbol_set = set;
-    ret->name = CORBA_string_dup (sym->name);
-    ret->expression = CORBA_string_dup (sym->expression);
-    ret->value = CORBA_string_dup (sym->value);
-    ret->expandable = sym->expandable;
-    ret->expanded = sym->expanded;
-    ret->children._type = sym->children._type;
-    ret->children._value = sym->children._value;
-    CORBA_any_set_release (&ret->children, CORBA_FALSE);
-    
-    return ret;
-}
-
-static GDF_Symbol *
-sym_search (GDF_SymbolSet *symset, gint id)
-{
-    int i;
-    GDF_Symbol *sym = NULL;
-
-    for (i = 0; i < symset->_length; i++) {
-	sym = &symset->_buffer[i];
-	if (sym->handle == id) {
-	    break;
-	}
-	sym = sym_search (sym->children._value, id);
-	if (sym) {
-	    break;
-	}
-    }
-    return sym;
-}
-
-
-static GDF_Symbol *
-find_sym_by_id (GdfLibdryadDebugger *dbg, 
-		GDF_SymbolSetId set, 
-		int sym_id)
-{
-    /* FIXME: This could be sped up by maintaining an id->sym lookup table */
-    GDF_SymbolSet *symset;
-    GDF_Symbol *ret;
-
-    symset = g_tree_lookup (dbg->sym_sets, GINT_TO_POINTER (set));
-    g_assert (symset);
-    
-    ret = sym_search (symset, sym_id);
-    return ret;
-}
-
-static void
-impl_expand_symbol (PortableServer_Servant servant,
-		    GDF_SymbolSetId set,
-		    CORBA_long id,
-		    CORBA_boolean expand,
-		    CORBA_Environment *ev)
-{
-    GDF_Symbol *sym;
-    GdfLibdryadDebugger *dbg = libdryad_debugger_from_servant (servant);
-
-    sym = find_sym_by_id (dbg, set, id);
-    g_return_if_fail (sym->expandable);
-    gdb_expand_symbol (dbg->gdb_inst, dbg, sym);
-    gdb_wait_for_prompt (dbg->gdb_inst);
+    return CORBA_Object_duplicate (bonobo_object_corba_objref (BONOBO_OBJECT (ss)), 
+				   ev);
 }
 
 static CORBA_Object
@@ -891,6 +703,7 @@ impl_get_event_channel (PortableServer_Servant servant,
 {
     CORBA_Object ret;
     GdfLibdryadDebugger *dbg = libdryad_debugger_from_servant (servant);
+    GDF_TRACE ();
     ret = bonobo_object_corba_objref (BONOBO_OBJECT (dbg->event_channel));
     return CORBA_Object_duplicate (ret, ev);
 }
@@ -940,12 +753,8 @@ init_libdryad_debugger_corba_class (void)
     libdryad_debugger_epv.step_over = impl_step_over;
     libdryad_debugger_epv.step_into = impl_step_into;
     libdryad_debugger_epv.step_out = impl_step_out;
+    libdryad_debugger_epv.get_locals = impl_get_locals;
     libdryad_debugger_epv.allocate_symbol_set = impl_allocate_symbol_set;
-    libdryad_debugger_epv.destroy_symbol_set = impl_destroy_symbol_set;
-    libdryad_debugger_epv.get_symbol_set = impl_get_symbol_set;
-    libdryad_debugger_epv.add_to_symbol_set = impl_add_to_symbol_set;
-    libdryad_debugger_epv.get_symbol = impl_get_symbol;
-    libdryad_debugger_epv.expand_symbol = impl_expand_symbol;
     libdryad_debugger_epv.get_event_channel = impl_get_event_channel;
     libdryad_debugger_epv.set_output_tty = impl_set_output_tty;
    
@@ -1134,7 +943,7 @@ connect_signals (GdfLibdryadDebugger *dbg)
 			execution_killed_cb, (gpointer) dbg);
     gdb_signal_connect (dbg->gdb_inst, "execution-exited",
 			execution_exited_cb, (gpointer) dbg);
-     gdb_signal_connect (dbg->gdb_inst, "execution-exited-signal",
+    gdb_signal_connect (dbg->gdb_inst, "execution-exited-signal",
 			execution_exited_signal_cb, (gpointer) dbg);
     gdb_signal_connect (dbg->gdb_inst, "execution-source-line",
 			execution_source_line_cb, (gpointer) dbg);
@@ -1146,14 +955,5 @@ connect_signals (GdfLibdryadDebugger *dbg)
 			error_cb, (gpointer)dbg);
 }
 
-int
-int_cmp (gconstpointer a, gconstpointer b) 
-{
-    return (GPOINTER_TO_INT (a) < GPOINTER_TO_INT (b) 
-            ? -1 
-            : (GPOINTER_TO_INT (a) == GPOINTER_TO_INT (b) ? 0 : 1));
-}
-
-
 
 
