@@ -8,6 +8,7 @@ typedef struct _GdfLibdryadDebugger GdfLibdryadDebugger;
 #include <bonobo/bonobo-object.h>
 #include "../../lib/gdf.h"
 #include "libdryad.h"
+#include "gdf-libdryad-symbol-set.h"
 
 BEGIN_GNOME_DECLS
 
@@ -30,8 +31,8 @@ struct _GdfLibdryadDebugger {
     GList *event_queue;
     GList *stopped_handlers;
 
-    GTree *sym_sets;
-    GTree *sym_set_sizes;
+    GdfLibdryadSymbolSet *locals;
+    GList *symsets;
 
     gboolean frame_changed;
 };
