@@ -790,8 +790,9 @@ execution_source_line_cb (GdfDebuggerClient *dbg,
 
 void
 execution_exited_cb (GdfDebuggerClient *dbg, gint exit_code,
-					 GdfSourceViewerManager *svm)
+                     GdfSourceViewerManager *svm)
 {
+    GDF_TRACE ();
 	if (svm->priv->prev_sv)
 		gdf_source_viewer_clear_current_line (svm->priv->prev_sv);
 	
