@@ -22,6 +22,7 @@
 
 #include <gnome.h>
 #include "gdf-variable-viewer.h"
+#include <gdf.h>
 
 struct _GdfVariableViewerPriv {
 	GtkWidget *ctree;
@@ -33,12 +34,13 @@ struct _GdfVariableViewerPriv {
     GTree *nodes;
 
 	/* signal identifiers */
-    guint symbol_set_changed_sig;
     guint symbol_changed_sig;
-    guint symbol_expanded_sig;
+    guint symbol_set_changed_sig;
 	guint program_unloaded_sig;
 
 	GdfDebuggerClient *dbg;
+    GdfSymbolSetClient *ss;
+    gboolean view_locals;
 };
 
 enum {
@@ -66,16 +68,17 @@ static void set_arg (GtkObject *object,
 static void create_children (GdfVariableViewer *vv);
 static void add_node (GdfVariableViewer *vv, GtkCTreeNode *parent, 
                       GDF_Symbol *sym);
-static void add_node_children (GdfVariableViewer *vv, GtkCTreeNode *parent, 
-                               GDF_Symbol *sym);
+static void add_dummy_child (GdfVariableViewer *vv, GtkCTreeNode *parent);
 static void clear_variable_viewer (GdfVariableViewer *vv);
 static void connect_debugger_signals (GdfVariableViewer *vv);
 static void disconnect_debugger_signals (GdfVariableViewer *vv);
+static void connect_symbol_set_signals (GdfVariableViewer *vv);
+static void disconnect_symbol_set_signals (GdfVariableViewer *vv);
 static void add_btn_clicked_cb (GtkWidget *btn, gpointer data);
 static void entry_activate_cb (GtkWidget *btn, gpointer data);
 static void tree_expand_cb (GtkWidget *ctree, GtkCTreeNode *node, 
                             gpointer data);
-static int int_cmp (gconstpointer a, gconstpointer b);
+static gint int_cmp (gconstpointer a, gconstpointer b);
 
 static GtkFrameClass *parent_class;
 
@@ -119,21 +122,39 @@ gdf_variable_viewer_new ()
 
 void 
 gdf_variable_viewer_view_symbol_set (GdfVariableViewer *vv,
-                                     int set_id)
+                                     GdfSymbolSetClient *ss)
 {
     g_return_if_fail (vv != NULL);
 	g_return_if_fail (GDF_IS_VARIABLE_VIEWER (vv));
 
-    vv->symbol_set = set_id;
+    vv->priv->view_locals = FALSE;
 
     if (vv->priv->dbg) {
-        if (set_id < 0) {
-            vv->symbol_set =
+        if (!ss) {
+            vv->priv->ss =
                 gdf_debugger_client_allocate_symbol_set (vv->priv->dbg);
+        } else {
+            vv->priv->ss = ss;
         }
-        
-        /* FIXME: Get the current symbol set */
     }
+    
+    /* FIXME: Get the current symbol set */
+}
+
+void 
+gdf_variable_viewer_view_locals (GdfVariableViewer *vv)
+{
+    g_return_if_fail (vv != NULL);
+	g_return_if_fail (GDF_IS_VARIABLE_VIEWER (vv));
+
+    vv->priv->view_locals = TRUE;
+
+    if (vv->priv->dbg) {
+        vv->priv->ss =
+            gdf_debugger_client_get_locals (vv->priv->dbg);
+    }
+    
+    /* FIXME: Get the current symbol set */
 }
 
 void 
@@ -158,15 +179,23 @@ gdf_variable_viewer_clear (GdfVariableViewer *vv)
 
 void
 gdf_variable_viewer_set_debugger (GdfVariableViewer *vv,
-                                   GdfDebuggerClient *dbg)
+                                  GdfDebuggerClient *dbg)
 {
 	g_return_if_fail (vv != NULL);
 	g_return_if_fail (GDF_IS_VARIABLE_VIEWER (vv));
 
+    gdf_variable_viewer_clear (vv);
+
 	if (vv->priv->dbg) {
 		disconnect_debugger_signals (vv);
 		gtk_object_unref (GTK_OBJECT (vv->priv->dbg));
-	}
+	} 
+
+    if (vv->priv->ss) {
+        disconnect_symbol_set_signals (vv);
+        gtk_object_unref (GTK_OBJECT (vv->priv->ss));
+        vv->priv->ss = NULL;
+    }
 	
 	vv->priv->dbg = dbg;
 
@@ -174,18 +203,15 @@ gdf_variable_viewer_set_debugger (GdfVariableViewer *vv,
         gtk_object_ref (GTK_OBJECT (dbg));
         if (GTK_OBJECT_FLOATING (GTK_OBJECT (dbg))) 
             gtk_object_sink (GTK_OBJECT (dbg));
-        
-        connect_debugger_signals (vv);
-    }
-    
-    if (dbg) {
-        if (vv->symbol_set < 0) {
-            vv->symbol_set = 
+        if (vv->priv->view_locals) {
+            vv->priv->ss = gdf_debugger_client_get_locals (vv->priv->dbg);
+        } else {
+            vv->priv->ss = 
                 gdf_debugger_client_allocate_symbol_set (vv->priv->dbg);
         }
-    } else {
-        /* FIXME: bug with restart */
-        vv->symbol_set = -1;
+
+        connect_debugger_signals (vv);
+        connect_symbol_set_signals (vv);
     }
 
     /* FIXME: View the symbol set */
@@ -236,12 +262,13 @@ variable_viewer_class_init (GdfVariableViewerClass *klass)
 void
 variable_viewer_init (GdfVariableViewer *vv)
 {
-    vv->symbol_set = -1;
 
     vv->priv = g_new0 (GdfVariableViewerPriv, 1);
 
 	vv->priv->dbg = NULL;
+    vv->priv->ss = NULL;
     vv->priv->nodes = NULL;
+    vv->priv->view_locals = FALSE;
 
 	create_children (vv);
 }
@@ -255,6 +282,8 @@ variable_viewer_destroy (GtkObject *obj)
 	
 	if (vv->priv->dbg)
 		gtk_object_unref (GTK_OBJECT (vv->priv->dbg));
+    if (vv->priv->ss)
+        gtk_object_unref (GTK_OBJECT (vv->priv->ss));
 
     if (vv->priv->nodes)
         g_tree_destroy (vv->priv->nodes);
@@ -359,29 +388,18 @@ create_children (GdfVariableViewer *vv)
 }
 
 void
-add_node_children (GdfVariableViewer *vv, GtkCTreeNode *parent, 
-                   GDF_Symbol *sym)
+add_dummy_child (GdfVariableViewer *vv, GtkCTreeNode *parent)
 {       
-    if (sym->expandable && !sym->expanded) {
-        GtkCTreeNode *node;
-        char *text[2];
-        text[0] = "";
-        text[1] = "";
-        
-        node = gtk_ctree_insert_node (GTK_CTREE (vv->priv->ctree), parent, 
-                                      NULL, text, 0, NULL, NULL, NULL, NULL, 
-                                      FALSE, FALSE);
-        gtk_ctree_node_set_row_data (GTK_CTREE (vv->priv->ctree), node, 
-                                     GINT_TO_POINTER (-1));
-    } else if (sym->expandable && sym->expanded) {
-        int i;
-        GDF_SymbolSet *children;
-        children = (GDF_SymbolSet*)sym->children._value;
-        GDF_TRACE_EXTRA ("%d", children->_length);
-        for (i = 0; i < children->_length; i++) {
-            add_node (vv, parent, &children->_buffer[i]);
-        }
-    }
+    GtkCTreeNode *node;
+    char *text[2];
+    text[0] = "";
+    text[1] = "";
+    
+    node = gtk_ctree_insert_node (GTK_CTREE (vv->priv->ctree), parent, 
+                                  NULL, text, 0, NULL, NULL, NULL, NULL, 
+                                  FALSE, FALSE);
+    gtk_ctree_node_set_row_data (GTK_CTREE (vv->priv->ctree), node, 
+                                 GINT_TO_POINTER (-1));
 }
 
 void
@@ -395,13 +413,13 @@ add_node (GdfVariableViewer *vv, GtkCTreeNode *parent, GDF_Symbol *sym)
     
     node = gtk_ctree_insert_node (GTK_CTREE (vv->priv->ctree), parent, NULL, 
                                   text, 0, NULL, NULL, NULL, NULL, 
-                                  FALSE, //sym->expandable ? FALSE : TRUE,
+                                  FALSE, 
                                   FALSE);
     g_tree_insert (vv->priv->nodes, GINT_TO_POINTER (sym->handle), node);
     gtk_ctree_node_set_row_data (GTK_CTREE (vv->priv->ctree), node, 
                                  GINT_TO_POINTER (sym->handle));
     if (sym->expandable) {
-        add_node_children (vv, node, sym);
+        add_dummy_child (vv, node);
     }
 }
 
@@ -416,18 +434,14 @@ clear_variable_viewer (GdfVariableViewer *vv)
 }
 
 static void
-symbol_set_changed_cb (GdfDebuggerClient *dbg,
-                       int set_id, GdfVariableViewer *vv)
+symbol_set_changed_cb (GdfSymbolSetClient *ss, GdfVariableViewer *vv)
 {
-    GDF_SymbolSet *syms;
+    GDF_SymbolSequence *syms;
     int i;
 
-    if (set_id != vv->symbol_set)
-        return;
-
     gdf_variable_viewer_clear (vv);
 
-    syms = gdf_debugger_client_get_symbol_set (dbg, set_id);
+    syms = gdf_symbol_set_client_get_symbols (ss);
     
     if (syms->_length > 0)
         vv->priv->nodes = g_tree_new (int_cmp);
@@ -441,66 +455,26 @@ symbol_set_changed_cb (GdfDebuggerClient *dbg,
 }
 
 static void
-symbol_changed_cb (GdfDebuggerClient *dbg, int symbol_set, int handle,
-                   GdfVariableViewer *vv)
+symbol_changed_cb (GdfSymbolSetClient *ss, int handle, GdfVariableViewer *vv)
 {
     GDF_Symbol *sym;
     GtkCTreeNode *node;
+    GtkCTreeNode *child;
     GtkCTreeRow *row;
 
-    if (vv->symbol_set != symbol_set)
-        return;
-
-    sym = gdf_debugger_client_get_symbol (dbg, symbol_set, handle);
+    sym = gdf_symbol_set_client_get_symbol (ss, handle);
 
     node = g_tree_lookup (vv->priv->nodes, GINT_TO_POINTER (handle));
     row = GTK_CTREE_ROW (node);
-    if (!row->children && sym->expandable) {
-        add_node_children (vv, node, sym);
-    }
-
     gtk_ctree_node_set_text (GTK_CTREE (vv->priv->ctree),
                              node, 1, sym->value);
 
-    /* FIXME: There is some sort of marshalling problem happening that causes
-     * CORBA_free() to intermittenly fail.  */
-    /* CORBA_free (sym); */
-}
-
-static void
-symbol_expanded_cb (GdfDebuggerClient *dbg, int symbol_set, int handle,
-                    GdfVariableViewer *vv)
-{
-    GDF_Symbol *sym;
-    GDF_SymbolSet *symset;
-    GtkCTreeNode *node;
-    GtkCTreeNode *child, *next;
-    int i;
-
-    if (vv->symbol_set != symbol_set)
-        return;
-
-    sym = gdf_debugger_client_get_symbol (dbg, symbol_set, handle);
-
-    node = g_tree_lookup (vv->priv->nodes, GINT_TO_POINTER (handle));
-
-    /* Remove the old/dummy children */
     child = GTK_CTREE_ROW (node)->children;
-    while (child) {
-        next = GTK_CTREE_ROW (child)->sibling;
-        gtk_ctree_remove_node (GTK_CTREE (vv->priv->ctree), child);
-        child = next;
-    }
-    
-    /* Add the new ones */
-    symset = sym->children._value;
-    for (i = 0; i < symset->_length; i++) {
-        add_node (vv, node, &symset->_buffer[i]);
+    if (sym->expandable && !child) {
+        add_dummy_child (vv, node);
     }
 
-    /* FIXME: There is some sort of marshalling problem happening that causes
-     * CORBA_free() to intermittenly fail.  */
-    /*CORBA_free (sym);*/
+    CORBA_free (sym);
 }
 
 void
@@ -511,21 +485,7 @@ connect_debugger_signals (GdfVariableViewer *vv)
 								   "program_unloaded",
 								   GTK_SIGNAL_FUNC (clear_variable_viewer),
 								   (gpointer)vv);
-    vv->priv->symbol_set_changed_sig =
-        gtk_signal_connect (GTK_OBJECT (vv->priv->dbg),
-                            "symbol_set_changed",
-                            GTK_SIGNAL_FUNC (symbol_set_changed_cb),
-                            (gpointer)vv);
-    vv->priv->symbol_changed_sig =
-        gtk_signal_connect (GTK_OBJECT (vv->priv->dbg),
-                            "symbol_changed",
-                            GTK_SIGNAL_FUNC (symbol_changed_cb),
-                            (gpointer)vv);
-    vv->priv->symbol_expanded_sig =
-        gtk_signal_connect (GTK_OBJECT (vv->priv->dbg),
-                            "symbol_expanded",
-                            GTK_SIGNAL_FUNC (symbol_expanded_cb),
-                            (gpointer)vv);
+
 }
 
 void
@@ -533,10 +493,30 @@ disconnect_debugger_signals (GdfVariableViewer *vv)
 {
 	gtk_signal_disconnect (GTK_OBJECT (vv->priv->dbg), 
 						   vv->priv->program_unloaded_sig);
-	gtk_signal_disconnect (GTK_OBJECT (vv->priv->dbg), 
+}
+
+void 
+connect_symbol_set_signals (GdfVariableViewer *vv)
+{
+    vv->priv->symbol_changed_sig =
+        gtk_signal_connect (GTK_OBJECT (vv->priv->ss),
+                            "symbol_changed",
+                            GTK_SIGNAL_FUNC (symbol_changed_cb),
+                            (gpointer)vv);
+    vv->priv->symbol_set_changed_sig =
+        gtk_signal_connect (GTK_OBJECT (vv->priv->ss),
+                            "symbol_set_changed",
+                            GTK_SIGNAL_FUNC (symbol_set_changed_cb),
+                            (gpointer)vv);
+}
+
+void
+disconnect_symbol_set_signals (GdfVariableViewer *vv)
+{
+	gtk_signal_disconnect (GTK_OBJECT (vv->priv->ss), 
 						   vv->priv->symbol_changed_sig);
-    gtk_signal_disconnect (GTK_OBJECT (vv->priv->dbg),
-                           vv->priv->symbol_expanded_sig);
+    gtk_signal_disconnect (GTK_OBJECT (vv->priv->ss),
+                           vv->priv->symbol_set_changed_sig);
 }
 
 static void
@@ -548,9 +528,8 @@ add_current_expression (GdfVariableViewer *vv)
         gtk_entry_get_text (GTK_ENTRY (gnome_entry_gtk_entry (GNOME_ENTRY (vv->priv->entry))));
     
     if (strcmp (text, "")) {
-        gdf_debugger_client_add_to_symbol_set (vv->priv->dbg, 
-                                                vv->symbol_set, 
-                                                text);
+        gdf_symbol_set_client_add_expression (vv->priv->ss, 
+                                              text);
     }
 
     gtk_entry_set_text (GTK_ENTRY (gnome_entry_gtk_entry (GNOME_ENTRY (vv->priv->entry))), "");
@@ -585,20 +564,34 @@ tree_expand_cb (GtkWidget *ctree, GtkCTreeNode *node, gpointer data)
     
     /* if this is a dummy node, sym_id will be -1 */
     if (sym_id == -1) {
+        int i;        
+        GDF_SymbolSequence *seq;
         sym_id = 
             GPOINTER_TO_INT (gtk_ctree_node_get_row_data (GTK_CTREE (ctree), 
                                                           node));
-        gdf_debugger_client_expand_symbol (vv->priv->dbg, vv->symbol_set,
-                                            sym_id, TRUE);
+        seq = gdf_symbol_set_client_get_symbol_children (vv->priv->ss, sym_id);
+
+        /* Remove the old/dummy children */
+        child = GTK_CTREE_ROW (node)->children;
+        while (child) {
+            GtkCTreeNode *next;
+            next = GTK_CTREE_ROW (child)->sibling;
+            gtk_ctree_remove_node (GTK_CTREE (vv->priv->ctree), child);
+            child = next;
+        }
+
+        for (i = 0; i < seq->_length; i++) {
+            add_node (vv, node, &(seq->_buffer[i]));
+        }
+
+        CORBA_free (seq);
     }
 }
 
-int
+gint
 int_cmp (gconstpointer a, gconstpointer b) 
 {
     return (GPOINTER_TO_INT (a) < GPOINTER_TO_INT (b) 
             ? -1 
             : (GPOINTER_TO_INT (a) == GPOINTER_TO_INT (b) ? 0 : 1));
 }
-
-
