@@ -60,9 +60,6 @@ enum {
     EXECUTION_KILLED,
     EXECUTION_EXITED,
     EXECUTION_SOURCE_LINE,
-    SYMBOL_SET_CHANGED,
-    SYMBOL_CHANGED,
-    SYMBOL_EXPANDED,
     STACK_FRAME_CHANGED,
     SIGNAL_RECEIVED,
     SIGNAL_TERMINATION,
@@ -86,7 +83,7 @@ gdf_debugger_client_construct (GdfDebuggerClient *dbg,
     CORBA_exception_init (&ev);
 
     channel = GDF_Debugger_get_event_channel (corba_object,
-                                               &ev);
+                                              &ev);
     
     /* FIXME: Check env */
 
@@ -745,130 +742,50 @@ gdf_debugger_client_step_out (GdfDebuggerClient *dbg)
     CORBA_exception_free (&ev);
 }
 
-GDF_SymbolSetId
+GdfSymbolSetClient *
+gdf_debugger_client_get_locals (GdfDebuggerClient *dbg)
+{
+    CORBA_Environment ev;
+    GdfSymbolSetClient *retval;
+    GDF_SymbolSet set;
+    
+    g_return_val_if_fail (dbg != NULL, NULL);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), NULL);
+   
+    CORBA_exception_init (&ev);
+    set = GDF_Debugger_get_locals (dbg->objref, &ev);
+
+    /* FIXME: Check env */
+    
+    retval = gdf_symbol_set_client_new_from_corba (set);
+
+    CORBA_exception_free (&ev);
+
+    return retval;
+}   
+
+GdfSymbolSetClient *
 gdf_debugger_client_allocate_symbol_set (GdfDebuggerClient *dbg)
 {    
     CORBA_Environment ev;
-    GDF_SymbolSetId retval;
+    GdfSymbolSetClient *retval;
+    GDF_SymbolSet set;
     
-    g_return_val_if_fail (dbg != NULL, -1);
-    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), -1);
+    g_return_val_if_fail (dbg != NULL, NULL);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), NULL);
    
     CORBA_exception_init (&ev);
-    retval = GDF_Debugger_allocate_symbol_set (dbg->objref,
-                                                &ev);
+    set = GDF_Debugger_allocate_symbol_set (dbg->objref, &ev);
 
     /* FIXME: Check env */
+
+    retval = gdf_symbol_set_client_new_from_corba (set);
     
     CORBA_exception_free (&ev);
 
     return retval;
 }
 
-void
-gdf_debugger_client_destroy_symbol_set (GdfDebuggerClient *dbg, 
-                                         GDF_SymbolSetId id)
-{
-    CORBA_Environment ev;
-    
-    g_return_if_fail (dbg != NULL);
-    g_return_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg));
-
-    CORBA_exception_init (&ev);
-    GDF_Debugger_destroy_symbol_set (dbg->objref, id, &ev);
-    
-    /* FIXME: Check env */
-    
-    CORBA_exception_free (&ev);
-}
-
-void 
-gdf_debugger_client_add_to_symbol_set (GdfDebuggerClient *dbg,
-                                        GDF_SymbolSetId id,
-                                        const char *expr)
-{
-    CORBA_Environment ev;
-    
-    g_return_if_fail (dbg != NULL);
-    g_return_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg));
-
-    CORBA_exception_init (&ev);
-    GDF_Debugger_add_to_symbol_set (dbg->objref, id, expr, &ev);
-    
-    /* FIXME: Check env */
-    
-    CORBA_exception_free (&ev);
-}
-
-GDF_SymbolSet *
-gdf_debugger_client_get_symbol_set (GdfDebuggerClient *dbg,
-                                     GDF_SymbolSetId id)
-{  
-    GDF_SymbolSet *retval;
-    CORBA_Environment ev;
-    
-    g_return_val_if_fail (dbg != NULL, NULL);
-    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), NULL);
-
-    CORBA_exception_init (&ev);
-    retval = GDF_Debugger_get_symbol_set (dbg->objref, id, &ev);
-    
-    /* FIXME: Check env */
-    
-    if (ev._major != CORBA_NO_EXCEPTION) {
-        CORBA_exception_free (&ev);
-        retval = NULL;
-    }
-
-    CORBA_exception_free (&ev);
-    return retval;    
-}
-
-GDF_Symbol *
-gdf_debugger_client_get_symbol (GdfDebuggerClient *dbg,
-                                 GDF_SymbolSetId set,
-                                 int handle)
-{
-    GDF_Symbol *ret;
-    CORBA_Environment ev;
-    
-    g_return_val_if_fail (dbg != NULL, NULL);
-    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), NULL);
-
-    CORBA_exception_init (&ev);
-    ret = GDF_Debugger_get_symbol (dbg->objref, set, 
-                                    (CORBA_long)handle, &ev);
-    
-    if (ev._major != CORBA_NO_EXCEPTION) {
-        CORBA_exception_free (&ev);
-        ret = NULL;
-    }
-
-    CORBA_exception_free (&ev);
-    return ret;
-}
-
-void 
-gdf_debugger_client_expand_symbol (GdfDebuggerClient *dbg, 
-                                    GDF_SymbolSetId id,
-                                    int handle,
-                                    gboolean expand)
-{
-    CORBA_Environment ev;
-    
-    g_return_if_fail (dbg != NULL);
-    g_return_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg));
-    
-    CORBA_exception_init (&ev);
-
-    GDF_Debugger_expand_symbol (dbg->objref, id, 
-                                 (CORBA_long)handle, 
-                                 expand ? CORBA_TRUE : CORBA_FALSE, &ev);
-    
-    /* FIXME: Check env */
-    CORBA_exception_free (&ev);
-}
-
 void 
 gdf_debugger_client_set_output_tty (GdfDebuggerClient *dbg,
                                      const char *tty_name)
@@ -1011,35 +928,6 @@ debugger_client_class_init (GdfDebuggerClientClass *klass)
                         GTK_TYPE_NONE, 2,
                         GTK_TYPE_STRING,
                         GTK_TYPE_INT);
-    debugger_client_signals [SYMBOL_SET_CHANGED] = 
-        gtk_signal_new ("symbol_set_changed",
-                        GTK_RUN_FIRST,
-                        object_class->type,
-                        GTK_SIGNAL_OFFSET (GdfDebuggerClientClass,
-                                           symbol_set_changed),
-                        gtk_marshal_NONE__INT,
-                        GTK_TYPE_NONE, 1,
-                        GTK_TYPE_INT);
-    debugger_client_signals [SYMBOL_CHANGED] = 
-        gtk_signal_new ("symbol_changed",
-                        GTK_RUN_FIRST,
-                        object_class->type,
-                        GTK_SIGNAL_OFFSET (GdfDebuggerClientClass,
-                                           symbol_changed),
-                        gtk_marshal_NONE__INT_INT,
-                        GTK_TYPE_NONE, 2,
-                        GTK_TYPE_INT,
-                        GTK_TYPE_INT);
-    debugger_client_signals [SYMBOL_EXPANDED] = 
-        gtk_signal_new ("symbol_expanded",
-                        GTK_RUN_FIRST,
-                        object_class->type,
-                        GTK_SIGNAL_OFFSET (GdfDebuggerClientClass,
-                                           symbol_expanded),
-                        gtk_marshal_NONE__INT_INT,
-                        GTK_TYPE_NONE, 2,
-                        GTK_TYPE_INT,
-                        GTK_TYPE_INT);
     debugger_client_signals [STACK_FRAME_CHANGED] = 
         gtk_signal_new ("stack_frame_changed",
                         GTK_RUN_FIRST,
@@ -1170,23 +1058,6 @@ emit_event_signal (GdfDebuggerClient *dbg,
                          debugger_client_signals[EXECUTION_SOURCE_LINE],
                          (gchar*)event->argument.loc_arg->file,
                          (gint)event->argument.loc_arg->line);
-    } else if (strcmp (event->event_name, "symbol_set_changed") == 0) {
-        g_assert (event->arg_type = GDF_EVENT_ARG_LONG);
-        gtk_signal_emit (GTK_OBJECT (dbg), 
-                         debugger_client_signals[SYMBOL_SET_CHANGED],
-                         (gint)event->argument.long_arg);
-    } else if (strcmp (event->event_name, "symbol_changed") == 0) {
-        g_assert (event->arg_type = GDF_EVENT_ARG_LONG_LIST);
-        gtk_signal_emit (GTK_OBJECT (dbg), 
-                         debugger_client_signals[SYMBOL_CHANGED],
-                         GPOINTER_TO_INT (event->argument.list_arg->data),
-                         GPOINTER_TO_INT (event->argument.list_arg->next->data));
-    } else if (strcmp (event->event_name, "symbol_expanded") == 0) {
-        g_assert (event->arg_type = GDF_EVENT_ARG_LONG_LIST);
-        gtk_signal_emit (GTK_OBJECT (dbg), 
-                         debugger_client_signals[SYMBOL_EXPANDED],
-                         GPOINTER_TO_INT (event->argument.list_arg->data),
-                         GPOINTER_TO_INT (event->argument.list_arg->next->data));
     } else if (strcmp (event->event_name, "frame_change") == 0) {
         g_assert (event->arg_type = GDF_EVENT_ARG_LONG);
         dbg->priv->cur_frame = event->argument.long_arg;
